# PRG - Exam #

This is the assignment for the PRG exam. Follow the instructions precisely and please note:

- If the module executes any functions when imported (e.g. print), points will be subtracted.
- If the module fails to be imported (due to syntactic errors), the solution is invalid.

The modules (packages) you implement should be packed in a zip file (files should be in the root of
the zip file, zipping directly the files, not the containing folder) and uploaded to BRUTE. For
every module, class and function you implement, do not forget to add a docstring. Code quality
(mainly following naming conventions and writing docstrings) is part of the grading of your
solution.


## Task 1 [3p] ##

Create a module named "traceback_code" which will produce the following traceback when executed.

```
Traceback (most recent call last):
  File "traceback_code.py", line 10, in <module>
    repeat_str(3, 4)
  File "traceback_code.py", line 6, in repeat_str
    raise TypeError("The type of argument 'text' is not 'str'")
TypeError: The type of argument 'text' is not 'str'
```


## Task 2 [6p] ##

Create a module named "task2_functions" and implement three functions according to the following
unit tests. You can copy the unit tests to the same module.

```
import unittest

class TestFunctions(unittest.TestCase):

    def test_swap_comma_dot(self):
        # [2p]
        self.assertEqual("23.145,00", swap_comma_dot("23,145.00"))
        self.assertEqual("23,145.00", swap_comma_dot("23.145,00"))
        with self.assertRaises(ValueError):
            swap_comma_dot("32")

    def test_repeated_elements(self):
        # [2p]
        self.assertEqual([2], repeated_elements([1, 2, 2, 3, 4]))
        self.assertEqual(["dog", "bird"], repeated_elements(["dog", "dog", "cat", "bird", "bird"]))
        self.assertEqual([], repeated_elements(["a", "b", "c", "d"]))

    def test_format_name_list(self):
        # [2p]
        self.assertEqual("Jenicek, Tomas\nNemy, Milan\n",
                         format_name_list([("Tomas", "Jenicek"), ("Milan", "Nemy")]))
        self.assertEqual("", format_name_list())
```


## Task 3 [1p] ##

Create a package with a single module containing the class definition below.

1. [1p] Choose the name of the package and module appropriately. Be sure to follow best practices.

```
class Evaluation:
    """The number of points (int) with a text evaluation (str) gotten from a particular assignment"""

    def __init__(self, points, text_evaluation):
        self.points = points
        self.text_evaluation = text_evaluation

    def __add__(self, other):
        return Evaluation(self.points + other, self.text_evaluation)

    def __repr__(self):
        return f"{self.points} ({self.text_evaluation})"
```


## Task 3.1 [5p] ##

In the module created in Task 3, implement the following tasks.

1. [1p] Add support for comparing instances using the equality operator "==".
2. [2p] Support for adding two Evaluation instances, following the example below
3. [2p] Write a unit test for the addition operator "+" and put it in a test case named
    "TestEvaluation".

Evaluation example

```
>>> Evaluation(3, "'==' done, '+' done") == Evaluation(1, "'==' done") + Evaluation(2, "'+' done")
True
```


## Task 3.2 [4p] ##

In the module created in Task 3, implement the following tasks.

1. [2p] Write function "save_evaluation" which takes two arguments - an Evaluation instance and
    a file path, and will store the data from the instance under the path. The format of the file is
    up to you.
2. [2p] Write function "load_evaluation" which takes a single argument - a file path, and returns
    an Evaluation instance containing the data previously saved by save_evaluation.


## Task 3.3 [6p] ##

In the module created in Task 3, implement the following tasks.

1. [2p] Implement class StudentRecord that stores a sequence of Evaluation instances.
2. [1p] Add support for comparing StudentRecord instances with '=='
3. [1p] Add support for adding Evaluation instance to a StudentRecord sequence.
4. [2p] Implement class PositiveEvaluation which inherits from Evaluation and ensures that the
    number of points is strictly positive (> 0). Ensure this by raising an appropriate exception if
    not. Try to reuse as much code from StudentRecord as possible.

StudentRecord example

```
>>> StudentRecord([Evaluation(4, "Nice work")]) == StudentRecord() + Evaluation(4, "Nice work")
True
```


## Epilogue ##

At the end of the exam, the following files and folders should be present in your zip file. They
should be in the root of the zip file, not in a nested folder.

- traceback_code.py - task 1
- task2_functions.py - task 2
- [folder of chosen name] - task 3
